@
|  DATASPEC.GSS:  procedures to compute spectra of observed data.
|
|     The data are assumed to be prefiltered (e.g., detrended).
|
|	Adoptation of GAUSS code written by Y. Wen.
|
|   This file contains:
|
|           spec = dataspec(y,nlags,nord);
|   where:  y is nobs x nvars
            nlags is number of lags in VAR
|           nord is number of ordinates over which to compute spectra
|                     (must be power of 2)
|
|   output is spec (k x nord). Each column is vech of spectral density
|           for relevant frequency.
|                       *********************
|   VAR.PRC: calculates VAR coefficients and VCV matrix of errors.
|   SPVAR.PRC: calculates spectral density matrix
|   SPTRAN.PRC: calculates spectral density matrix of differenced data
@

proc (2) = var(y,nlags);
    local nobs, ndep, x, i, beta, e;
    nobs = (rows(y));
    ndep = nobs - nlags;
    x=ones(ndep,1);


    i=1;
    do while i<=nlags;
        x = x~y[nlags+1-i:nobs-i,.];
        i=i+1;
    endo;

    y=y[nlags+1:nobs,.];
    beta = invpd(x'x)*x'y;
    e = y-x*beta;
    retp(beta[2:rows(beta),.]',e'e/ndep);
endp;


proc spvar(b,seps,w);
    local im, z, z1, sm, smi, ss, nvar, nlags, ii, n1;

    nvar = rows(seps);       @ Number of Variables in VAR @
    nlags = cols(b)/(nvar);

@ -- Calculate Spectrum of S --- @
    let im = 0+1i;
    z = exp(-w*im);
    z1 = 1;
    sm = eye(nvar);
    ii=1;
    do while ii <= nlags;
        n1 = nvar*(ii-1);
        z1 = z*z1;
        sm = sm-z1*b[.,n1+1:n1+nvar];
        ii=ii+1;
    endo;
    smi = inv(sm);
    ss = smi*seps*smi';
@ ss=ss/(2*pi);  @

retp(ss);
endp;


proc sptran(spec,w);
    local im, z;
    let im = 0+1i;
    z = exp(-w*im);
          @ use first difference filter @
    spec = (1-z)*spec*(1-z)';
    retp(spec);
endp;

proc dataspec(y,nlags,nord);
    local beta, seps, nt, svdat, step, wvec, j, ss, ssdif;

@  Estimate VAR  @
    {beta, seps} = var(y,nlags);

@  Build spectra  @
    nt = rows(seps);
    svdat = zeros(nt*(nt+1)/2,nord);
    step = pi/nord;
    wvec = seqa(step,step,nord);
    j=1;
    do while j<=nord;
        ss = spvar(beta,seps,wvec[j,.]);  @ Spectrum of VAR @
        ssdif = sptran(ss,wvec[j,.]);     @ Spectrum of differenced data  @
        svdat[.,j] = vech(ssdif);
        j=j+1;
    endo;

retp(svdat);
endp;



