/*  ct.prc  */

/* Estimates common trend for set of variables via OLS. 

FORMAT: {bet, yhat, fstat, pval} = ct(y,tsi); 

INPUTS: y, TxN matrix of variables. T=observations, N=# of variables. 
        tsi, 1x1 indicator variable.    tsi=1: inputted data are in levels, estimate linear trend.
                                        tsi=2: inputted data are differenced, estimate common growth rate 
OUTPUT: 	bet:    2xN matrix of restricted OLS estimates, intercept and slope if tsi = 1;
                        1xN vector of restriced OLS estimates of common growth rate.
		yhat:   TxN matrix of fitted trends
                fstat:  f stat of null that the restriction holds
                pval:   pvalue associated with tstat

Restrictions of the form Rxbeta = r 
Reference: Amemiya (1985 Advanced Econometrics, Sections 1.4 and 1.5) */

proc(4) = ct(y,tsi);

local T,N,bigr,littler,i,bigy,determ,subdeterm,xxxdet,bun,bres,yhat,resid,sighat,
        fstat,pval,yret,bret;
	
	T = rows(y);
	N = cols(y);


if tsi == 1;

/*  Establish restrictions  */

	bigr = zeros(N-1,2*N);
	bigr[.,2] = ones(N-1,1);
	i = 2; do while i<=N;
		bigr[i-1,2*i] = -1;
	i = i+1; endo;
	bigr = bigr';
	littler = zeros(N-1,1);

/* establish vector of dependent variables  */

	bigy = vec(y);

/* establish matrix of explanatory variables */

	determ = zeros(T*N,2*N);
	subdeterm = ones(T,1)~seqa(1,1,T);
	i=1; do while i<=N; 
		determ[(T*(i-1)+1):(T*i),(2*(i-1)+1):(2*i)] = subdeterm;
	i=i+1; endo;

	xxxdet = invpd(determ'determ);
	bun = xxxdet*(determ'bigy);

	bres = bun - (xxxdet*bigr)*invpd(bigr'xxxdet*bigr)*(bigr'bun - littler);
	yhat = determ*bres;
    resid = bigy-yhat;
    sighat = (1/(T-2))*resid'resid;
    fstat = (1/(sighat*(N-1)))*( (bigr'bun-littler)'(invpd(bigr'xxxdet*bigr))*(bigr'bun-littler) );
    pval = cdffc(fstat,N-1,T-2);

	yret = zeros(T,N);
	bret = zeros(N,2);
	i = 1; do while i<= N;
		bret[i,.] = bres[2*(i-1)+1:2*i]';
		yret[.,i] = yhat[T*(i-1)+1:T*i];
	i = i+1; endo;

retp(bres,yret,fstat,pval);

elseif tsi==2;

/*  Establish restrictions  */

	bigr = zeros(N-1,N);
	bigr[.,1] = ones(N-1,1);
	i = 2; do while i<=N;
		bigr[i-1,i] = -1;
	i = i+1; endo;
	bigr = bigr';
	littler = zeros(N-1,1);

/* establish vector of dependent variables  */

	bigy = vec(y);

/* establish matrix of explanatory variables */

	determ = zeros(T*N,N);
	subdeterm = ones(T,1);
	i=1; do while i<=N; 
		determ[(T*(i-1)+1):(T*i),i] = subdeterm;
	i=i+1; endo;

	xxxdet = invpd(determ'determ);
	bun = xxxdet*(determ'bigy);

	bres = bun - (xxxdet*bigr)*invpd(bigr'xxxdet*bigr)*(bigr'bun - littler);
	yhat = determ*bres;
    resid = bigy-yhat;
    sighat = (1/(T-1))*resid'resid;
    fstat = (1/(sighat*(N-1)))*( (bigr'bun-littler)'(invpd(bigr'xxxdet*bigr))*(bigr'bun-littler) );
    pval = cdffc(fstat,N-1,T-1);

	yret = zeros(T,N);
	bret = zeros(N,1);
	i = 1; do while i<= N;
		bret[i,.] = bres[i];
		yret[.,i] = yhat[T*(i-1)+1:T*i];
	i = i+1; endo;

retp(bres,yret,fstat,pval);

else;

 retp("tsi must be either 1 or 2",0,0,0);

endif;

endp;

