/* MULTIT.PRC:		PROCEDURE THAT GENERATES DRAWINGS FROM A MULTIVARIATE-T DENSITY  */

/* 

/* k-variate kernal: [df + (gam - mu)'V*(gam - mu)]^-(k+2/df) */

format:		{ betadraw, sig, fcnval }  = multit(mu,sig,v,swish,df,normcons);

inputs:		mu: mean of density
		sig: V = invpd(sig);
		V
		swish: chol(sig)'
		df: degrees of freedom
		normcons: -0.5*ln(det(sig)) -- normalizing constant for evaluating the t distribution

outputs:	betadraw: parameter drawing
		sig: scaling factor for second-moment matrix
		fcnval: value of t distribution evaluted at parameter drawing

*/

proc(3) = multit(mu,sig,v,swish,df,normcons);

		local ranw,chi,sigu,k,draw1,shck,draw,zzz,twght;

	/*  draw parameters  */

	ranw = rndn(df,1);
	chi = ranw'ranw;
	sigu = (chi/df)^(-0.5);
	k = rows(mu);
	draw1 = rndn(k,1);
	shck = sigu*swish*draw1;
	draw = mu + shck;
        zzz = (draw-mu)'V*(draw-mu);
        twght = -0.5*(df+k-1)*ln(df+zzz) + normcons;

	retp(draw,sigu,twght);

endp;