/* niw.prc: procedure designed to generate drawings of parameters from a normal-inverted wishart distribution
		(or normal-inverted gamma if vmat is 1x1) */
/*
format:		{betadraw1, betadraw2,wish} = niw(betahat,sxx,sinv,df);

inputs:		betahat: ols estimate -- xxx*(x'y), where xxx = invpd(x'x)
		sxx: cholesky decomposition of xxx -- chol(xxx)'
		sinv: cholesky decomposition of inv(vmat) -- chol(inv(vmat)), where vmat = (1/df)*(res'res),
										and res = y - xbetahat
		df: degrees of freedom 

output:		antithetic pair of drawings, drawing of vcv matrix from wishart 
*/

proc(3) = niw(betahat,sxx,sinv,df);

	local nvars,nparams,ranw,rantr,wish,swish,kron,ranc,shock,betadraw1,betadraw2;

    	NVARS = ROWS(SINV);
        NPARAMS = ROWS(BETAHAT);
        RANW = RNDN(NVARS,DF)/SQRT(DF);
        RANTR = RANW'SINV;
        WISH = INV(RANTR'RANTR);
        SWISH = (CHOL(WISH))';
        KRON = SWISH.*.SXX;
        RANC = RNDN(NPARAMS,1);
        SHOCK = KRON*RANC;
        BETADRAW1 = BETAHAT + SHOCK;
    	BETADRAW2 = BETAHAT - SHOCK;

	RETP(BETADRAW1,BETADRAW2,WISH);

ENDP;