/*  spec_arma.prc  */

/* Estimates spectrum of a time series based on estimates obtained from an arma(p,q) model */
/* Inputs: 	y, Tx1 variable. T=observations. 
		p, # of ar terms.
		q, # of ma terms.
		r, # of frequency points
*/

/* Format: { coefs, sp, fr } = spec_arma(y,p,q,r); */
/* Output: 	coefs: ARMA estimates
		    sp: spectral density ordinates
       		fr: frequencies (measured as cycles per period)
*/

proc(3) = spec_arma(y,p,q,r);

local nobs,coefs,ll,e,vcb,aic,sbc,sig,a,b,u,c;

	nobs=rows(y);
	{coefs,ll,e,vcb,aic,sbc} = arima(0,y,p,0,q,0);
	
	sig = (1/(nobs-p-q))*e'e;
	a = coefs[1:p];
	b = coefs[p+1:p+q];
	
	u = (pi*seqa(1/r,1/(r),r)).*.seqa(1,1,maxc(p|q))';  /* range is [0, pi] */

	c = (1-cos(u[.,1:q])*b)^2 + (sin(u[.,1:q])*b)^2  ;

	c = (sig/(2*pi))*c./( (1-cos(u[.,1:p])*a)^2 + (sin(u[.,1:p])*a)^2 );

retp(coefs, c, (u[.,1]/(2*pi)));

endp;